#!/bin/bash -e
########################################################################
# build_pygccxml.sh
########################################################################

####################################
# Useful variables
prog=${0##*/}
thisdir=$(cd "$(dirname "${0}")" && pwd -P)

####################################
# Useful functions.

# Report usage.
usage() {
   echo "USAGE: ${prog} <product_directory> <p2713d|p2714|p2714b|p2715a|p382> [tar]"
}

# Make sure we have UPS and ssibuildshims set up.
get_ssibuildshims() {
  [[ -n "${UPS_DIR}" ]] || { echo "ERROR: UPS not set up" 1>&2; exit 1; }
  source $("${UPS_DIR}/bin/ups" setup ${SETUP_UPS})
  setup ssibuildshims ${ssibuildshims_version} -z ${product_dir} || { \
    echo "ERROR: failed to set up ssibuildshims ${ssibuildshims_version}" 1>&2; \
    exit 1; }
}

# Handle qualifiers.
set_qualifiers() {
  local qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
  local qual
  for qual in "${qlist[@]}"; do
    case $qual in
      p2713d|p2714|p2714b|p2715a|p372|p382) pqual=$qual;;
      *) ssi_die "unrecognized qualifier $qual"
    esac
  done
  [[ -n "$pqual" ]] || ssi_die "Python qualifier is mandatory (e.g. p382)"
}


# Make the distribution and leave if successful.
maybe_make_distribution() {
  local make_tarball_opts=( \
    -d bin:lib/python${pythonMdotm}:man \
    -F "${product_dir}" "${package}" "${pkgver}" "${basequal}" NO_FLAVOR \
    )
  if [ "${maketar}" = "tar" ] && \
    [ -d "${site_packages_dir}" ] && \
    [ -f "${product_dir}/${package}/${pkgver}.version/NULL_${pqual}" ]; then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball "${make_tarball_opts[@]}" 
	  exit 0
  fi
}

########################################################################
# Main.
########################################################################

####################################
# Arguments.
product_dir=${1}
basequal=${2}
maketar=${3}

####################################
# Basic checks.
[[ -n "${product_dir}" ]] || \
  { printf "ERROR: please specify the local product directory\n$(usage)\n" 1>&2; \
  exit 1; }

if [[ -z "${basequal}" ]] || [[ "${basequal}" == "tar" ]]; then
  printf "ERROR: Python qualifier is mandatory\n$(usage)\n" 1>&2
  exit 1
fi
####################################

####################################
# Package name and version information.
package=pygccxml
origpkgver=v1_9_1
pkgver=${origpkgver}b
ssibuildshims_version=v2_00_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}
####################################

# Obtain access to useful functions.
get_ssibuildshims
source ssi_functions

# Deal with qualifiers.
set_qualifiers

# Define globals.
source define_basics NO_FQ_DIR NO_DEBUG_SOURCE

# Useful information for dealing with Python
if [[ "${pqual}" =~ ^p(2)(7)([0-9]*)(.*)$ ]] || \
  [[ "${pqual}" =~ ^p(3)([78])([0-9]*)(.*)$ ]]; then
  pythonMm="${BASH_REMATCH[1]}${BASH_REMATCH[2]}"
  pythonMdotm="${BASH_REMATCH[1]}.${BASH_REMATCH[2]}"
  pythonDotVersion="${BASH_REMATCH[1]}.${BASH_REMATCH[2]}.${BASH_REMATCH[3]}"
  pythonUpsVersion="v${pythonDotVersion//./_}${BASH_REMATCH[4]}"
  site_packages_dir="${pkgdir}/lib/python${pythonMdotm}/site-packages/"
else
  ssi_die "unable to extract major and minor versions from Python qualifier ${pqual}"
fi

# Possible short-circuit.
maybe_make_distribution

echo "building ${package} for Python ${pythonDotVersion} (${pythonUpsVersion})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

####################################
# Ensure directories.
mkdir -p "${blddir}" || ssi_die "failed to create build directory ${blddir}"
####################################

####################################
# Declare ourselves privately to allow setup of dependencies.
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product \
  "${product_dir}" ${package} ${pkgver} ${basequal} NO_FLAVOR

setup -B ${package} ${pkgver} -q+${basequal} \
  -z "${fakedb}:${product_dir}:${PRODUCTS}" || \
  ssi_die "fake setup failed: missing dependency or qualifier not supported by table file?"
####################################

# Print active products.
echo
ups active
echo

# Work in a build directory, which will not be part of any distribution.
cd ${blddir} || ssi_die "unable to cd into ${blddir}"

####################################
# Install, and test installation.
set -x

# Unpack and get ready.
tar xf ${tardir}/${pkgtarname}.tar.gz || \
  ssi_die "failed to untar ${pkgtarname}.tar.gz"
cd ${package}-${pkgdotver} || \
  ssi_die "failed to cd into ${package}-${pkgdotver}"
mkdir -p ${site_packages_dir} || \
  ssi_die "Failed to create the site_packages directory ${site_packages_dir}"

# Install.
python setup.py install --prefix ${pkgdir} || \
  ssi_die "python setup.py install failed"

# Smoke test installation.
python -c "import pygccxml" || \
  ssi_die "Python failed to find pygccxml package"

set +x

# Verify install.
[[ -d "${site_packages_dir}" ]] || \
  ssi_die "failed to create ${site_packages_dir}:\n" \
  "      something is very wrong"
####################################

# Declare the installed package.
${SSIBUILDSHIMS_DIR}/bin/declare_product "${product_dir}" \
  ${package} ${pkgver} ${basequal} NO_FLAVOR || \
  ssi_die "failed to declare ${package} ${pkgver}"

echo "Finished building ${package} ${pkgver}"

# Attempt to set up the package.
setup -B ${package} ${pkgver} -q ${basequal} \
  -z "${product_dir}:${PRODUCTS}" || \
  ssi_die "setup of installed package failed"

echo "${package} is installed at ${PYGCCXML_DIR}"

# This must be last.
maybe_make_distribution

exit 0
